// BSP : STM32 boards groups by STMicroelectronics 2.7.1
// Borad select : STM Boards groups / LoRa boars
// Board part number : LoRa-E5 mini (Board to be selected from Tools submenu 'Board part number)

// 2024/02/05

#include <Adafruit_GFX.h>         // 1.11.9
#include <Adafruit_SSD1306.h>     // 2.5.9
#include "myFonts/tahoma5pt7b.h"  // any font
#include "myFonts/tahoma7pt7b.h"

// Divider circuit for Battery voltage reading,  Q1-Drain(BAT+) -- 1000k -- A4 -- 510k -- D0
#define PIN_BATT       A4       // battery voltage
#define READ_ENABLE    D0       // battery voltage ADC LOW:enable, HIGH:disable

// Oled display instance
Adafruit_SSD1306 display(128, 64, &Wire, -1);

// RM0461 Reference manual Rev 8  37.1.4 IEEE 64-bit unique device ID register (UID64)
uint32_t uid = *((uint32_t*)0x1FFF7580);    // use uid64 dor Master device UID

// *******************************************************************************************************
void setup() {
  Serial.begin(115200);
  while(!Serial);
  delay(2000);
  Serial.println("LoRa-E5 Vbatt calibration");

  // Pin initialization
  pinMode(LED_RED, OUTPUT);
  pinMode(READ_ENABLE, OUTPUT);

  digitalWrite(LED_RED, HIGH);
  digitalWrite(READ_ENABLE, LOW);   // read enable

  // ADC initialization
  analogReadResolution(12);

  // Display initialization
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setFont(&tahoma7pt7b);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 15);
  display.println("LoRa-E5");
  display.setCursor(0, 31);
  display.println("Vbatt Calibration");
  display.setCursor(0, 47);
  display.println(uid, HEX);  
  display.display();
  delay(2000);
  display.clearDisplay();

}

// ***********************************************************************************************************
void loop() 
{
  digitalWrite(LED_RED, LOW);

  uint32_t Vadc = 0;
  for(int i = 0; i < 256; i++) {
    Vadc = Vadc + analogRead(PIN_BATT);
  }
  Vadc = Vadc / 256;

  // device #1(0x05358291) map()data
//  int adc_3400 = 1326;  // ADC data when battery voltage is 3.4V
//  int adc_4200 = 1509;  // ADC data when battery voltage is 4.2V
  // device #2(0x0535EE2F) map()data
//  int adc_3400 = 1347;  // ADC data when battery voltage is 3.4V
//  int adc_4200 = 1525;  // ADC data when battery voltage is 4.2V
  // device #3(0x05370378) map()data
  int adc_3400 = 1442;  // ADC data when battery voltage is 3.4V
  int adc_4200 = 1633;  // ADC data when battery voltage is 4.2V
  float Vbatt = map(Vadc, adc_3400, adc_4200, 3400, 4200) / 1000.0;   // Converts to battery voltage 3.4 ~ 4.2V

  Serial.print(Vadc);
  Serial.print("    ");
  Serial.print(Vadc, HEX);
  Serial.print("    ");
  Serial.println(Vbatt, 3);


    display.clearDisplay();
    display.setCursor(0, 15); display.print("UID  ");
    display.setCursor(32, 15); display.print(uid, HEX);    
    display.setCursor(0, 31); display.print(Vadc);    
    display.setCursor(0, 47); display.print(Vadc, HEX);   
    display.setCursor(0, 63); display.print(Vbatt, 3); display.print(" V");   
    display.display();


  digitalWrite(LED_RED, HIGH);

  delay(2000);
}
